﻿package monkeykong.gamestate
{
    import flash.display.DisplayObject;
    import flash.geom.Rectangle;
    import monkeykong.G;
    import monkeykong.ingame.Floor;
    import monkeykong.ingame.LevelBuilder;
    import monkeykong.ingame.MarioClone;
    import monkeykong.ingame.Nanner;
    import monkeykong.ingame.Wall;
    import stego.collision.CollisionEvent;
    import stego.collision.CollisionInfo;
    import stego.core.GameInterface;
    import stego.core.GameObject;
    import stego.core.GameObjectHolder;
    import stego.core.GameState;
    import stego.script.Step;
    import stego.script.step.Animate;
    import stego.script.step.Loop;
    import stego.script.step.RunFunc;
    import stego.Sounds;
    import stego.Util;
    import stego.util.SceneLayers;

    public class Gameplay extends GameState
    {
        private var backgroundLayer:GameObjectHolder;
        private var platformLayer:GameObjectHolder;
        private var enemiesLayer:GameObjectHolder;
        private var playerLayer:GameObjectHolder;
        private var itemLayer:GameObjectHolder;

        private var player:MarioClone;

        private var levelBuilder:LevelBuilder;

        private var leftSideChangeScript:Step;
        private var rightSideChangeScript:Step;
        private var leftSideCurtain:GameObject;
        private var rightSideCurtain:GameObject;

        public override function Begin(game:GameInterface):void
        {
            super.Begin(game);

            Game.Physics.gravity = 230;

            /// put camera in position
            Game.cameraX = 0.5 * G.GameScreenWidth;
            Game.cameraY = 0.5 * G.GameScreenHeight;

            // HACKS - currently the physics util still assumes
            // it knows what each layer is for so this is FRAGILE
            backgroundLayer = Game.Layers.layers[0];
            enemiesLayer = Game.Layers.layers[1];
            platformLayer = Game.Layers.layers[2];
            playerLayer = Game.Layers.layers[3];
            itemLayer = Game.Layers.layers[4];

            levelBuilder = new LevelBuilder(
                backgroundLayer,
                platformLayer,
                enemiesLayer,
                playerLayer,
                itemLayer );

            CreatePlayer();
            CreateLevel();

            stego.Sounds.PlaySound(monkeykong.Sounds.screenchangeSound);

            game.Cheats.AddCheatCode('!', ChangeLeftSide );
            game.Cheats.AddCheatCode('@', ChangeRightSide );

            SetupTransitions();
        }

        private function SetupTransitions():void
        {
            leftSideCurtain = Game.Layers.uiLayer.Add();
            leftSideCurtain.graphics.beginFill(0x181860);
            leftSideCurtain.graphics.drawRect(0, 0, 0.5 * G.GameScreenWidth, G.GameScreenHeight);
            leftSideCurtain.graphics.endFill();
            leftSideCurtain.x = -leftSideCurtain.width;

            rightSideCurtain = Game.Layers.uiLayer.Add();
            rightSideCurtain.graphics.beginFill(0x181860);
            rightSideCurtain.graphics.drawRect(0, 0, 0.5 * G.GameScreenWidth, G.GameScreenHeight);
            rightSideCurtain.graphics.endFill();
            rightSideCurtain.x = G.GameScreenWidth + rightSideCurtain.width;

            leftSideChangeScript = new Loop(1, [ new Animate(leftSideCurtain, { x: -0.5 * G.GameScreenWidth }, { x:0 }, 0.25),
                                                 new RunFunc(levelBuilder.RecreateLeftSide),
                                                 new Animate(leftSideCurtain, { x:0 }, { x: -0.5 * G.GameScreenWidth }, 0.25) ]);
            rightSideChangeScript = new Loop(1, [ new Animate(rightSideCurtain, { x:G.GameScreenWidth }, { x: 0.5 * G.GameScreenWidth }, 0.25),
                                                  new RunFunc(levelBuilder.RecreateRightSide),
                                                  new Animate(rightSideCurtain, { x: 0.5 * G.GameScreenWidth }, {x:G.GameScreenWidth}, 0.25) ]);
        }

        private function CreatePlayer():void
        {
            player = playerLayer.Add(MarioClone) as MarioClone;
            player.x = 10 * G.TILE_SIZE;
            player.y = 16 * G.TILE_SIZE;
            player.inputUtil = Game.inputUtil;
        }

        private function CreateLevel():void
        {
            levelBuilder.RecreateLeftSide();
            levelBuilder.RecreateRightSide();
        }

        public override function End():void
        {
            Game.Cheats.RemoveCheatCode('!', levelBuilder.RecreateLeftSide );
            Game.Cheats.RemoveCheatCode('@', levelBuilder.RecreateRightSide );

            levelBuilder.Shutdown();
            Game.Layers.RemoveAll();
            super.End();
        }

        public override function Update():void
        {
            super.Update();

            SceneLayers.ForEachCartesianProductStyle(playerLayer, itemLayer, MonkeyToBananaCollision);
        }

        private function MonkeyToBananaCollision(monkey:GameObject, banana:GameObject):void
        {
            var ci:CollisionInfo = Util.CheckAabbCollision(monkey, banana, Game.Time.TimeDelta);

            if (null != ci)
            {
                if (banana is Nanner)
                {
                    banana.isAlive = false;
                    if (banana.x < 0.5 * G.GameScreenWidth)
                    {
                        ChangeRightSide();
                    }
                    else
                    {
                        ChangeLeftSide();
                    }
                }
            }
        }
        private function ChangeLeftSide():void
        {
            script = leftSideChangeScript;
            script.Restart();
            stego.Sounds.PlaySound(monkeykong.Sounds.nannerSound);
        }
        private function ChangeRightSide():void
        {
            script = rightSideChangeScript;
            script.Restart();
            stego.Sounds.PlaySound(monkeykong.Sounds.nannerSound, null, 0.5);
        }
    }
}
