﻿package monkeykong.gamestate
{
    import flash.display.DisplayObject;
    import flash.display.Sprite;
    import flash.events.MouseEvent;
    import flash.utils.getTimer;
    import monkeykong.G;
    import monkeykong.Sounds;
    import stego.constants.Colors;
    import stego.core.GameInterface;
    import stego.core.GameObject;
    import stego.core.GameObjectHolder;
    import stego.core.GameState;
    import stego.script.Step;
    import stego.script.step.Condition;
    import stego.script.step.Loop;
    import stego.script.step.PlaySound;
    import stego.script.step.RunFunc;
    import stego.script.step.WaitTime;
    import stego.Util;
    import stego.widget.Fader;
    import monkeykong.titlescreen.Brick;

    public class Title extends GameState
    {
        [Embed(source='../../../assets/title.png')]
        private static const TITLE_GRAPHIC:Class;
        [Embed(source='../../../assets/titlebutton.png')]
        private static const TITLE_BUTTON_GRAPHIC:Class;

        private var clickToStartText:GameObject;
        private var mouseDummy:GameObject;

        private var fader:Fader;
        private var uiLayer:GameObjectHolder;
        private var brickLayer:GameObjectHolder;

        public override function Begin(game:GameInterface):void
        {
            super.Begin(game);

            script = new Loop(1, [
                        new PlaySound(Sounds.screenchangeSound),
                        new PlaySound(Sounds.wallbreakSound),
                        new RunFunc(FadeIn),
                        new Loop( -1, [
                                        new WaitTime(0.75),
                                        new RunFunc(ToggleStartText) ])] );

            /// put camera in position
            Game.cameraX = 0.5 * G.GameScreenWidth;
            Game.cameraY = 0.5 * G.GameScreenHeight;

            uiLayer = Game.Layers.layers[4];
            brickLayer = Game.Layers.layers[5];

            var titleImage:GameObject = uiLayer.Add(GameObject);
            titleImage.addChild(new TITLE_GRAPHIC());
            titleImage.PhysicsEnabled = false;
            titleImage.AffectedByGravity = false;

            clickToStartText = uiLayer.Add(GameObject);
            clickToStartText.addChild(new TITLE_BUTTON_GRAPHIC());
            clickToStartText.PhysicsEnabled = false;
            clickToStartText.AffectedByGravity = false;

            mouseDummy = uiLayer.Add(GameObject);
            mouseDummy.graphics.beginFill(Colors.MAGENTA, 0.0);
            mouseDummy.graphics.drawRect(0, 0, G.GameScreenWidth, G.GameScreenHeight);
            mouseDummy.graphics.endFill();
            mouseDummy.addEventListener(MouseEvent.CLICK, StartGame);
            mouseDummy.PhysicsEnabled = false;
            mouseDummy.AffectedByGravity = false;

            fader = uiLayer.Add(Fader) as Fader;
            fader.FadeToColor(0, Colors.BLACK);

            AddBricks();
        }

        private function ToggleStartText():void
        {
            clickToStartText.visible = !clickToStartText.visible
        }

        private function StartGame(e:MouseEvent):void
        {
            Game.ChangeGameState(new Gameplay());
        }

        private function FadeIn():void
        {
            fader.FadeBack(0.125);
            Game.Music.StartNewMusic();
        }

        private function AddBricks():void
        {
            var curX:Number = 0;
            var curY:Number = -4;

            var brickWidth:Number = 16;
            var brickHeight:Number = 8;
            var brickStartX:Number = 0;

            while (curY < G.GameScreenHeight)
            {
                while (curX < G.GameScreenWidth)
                {
                    var brick:GameObject = brickLayer.Add(Brick);
                    brick.x = curX;
                    brick.y = curY;
                    curX += brickWidth;
                }
                curY += brickHeight;

                // every other row is shifted over slightly
                brickStartX = (brickStartX - (0.5 * brickWidth)) % brickWidth
                curX = brickStartX;
            }
        }

        override public function Update():void
        {
            super.Update();

            LaunchABrick();
            LaunchABrick();
            LaunchABrick();
        }

        private function LaunchABrick():void
        {
            var bricks:/*Brick*/Array = brickLayer.Objects;
            if (0 < bricks.length)
            {
                var brick:Brick = Util.RandomOneOf(brickLayer.Objects);
                if (!brick.hasBeenLaunched)
                {
                    brick.Launch();
                }
            }
        }

        public override function End():void
        {
            mouseDummy.removeEventListener(MouseEvent.CLICK, StartGame);
            Game.Layers.RemoveAll();
            super.End();
        }
    }
}
