﻿package monkeykong.ingame
{
    import flash.display.DisplayObject;
    import flash.geom.Rectangle;
    import monkeykong.G;
    import stego.core.GameObject;
    import stego.core.GameObjectHolder;
    import stego.Util;

    /**
     * class to rebuild levels for monkey kong
     */
    public class LevelBuilder
    {
        private var backgroundLayer:GameObjectHolder;
        private var platformLayer:GameObjectHolder;
        private var enemiesLayer:GameObjectHolder;
        private var playerLayer:GameObjectHolder;
        private var itemLayer:GameObjectHolder;

        private var levelNumber:Number;
        private var leftSideBuildingFuncs:/*Function*/Array;
        private var rightSideBuildingFuncs:/*Function*/Array;

        public function LevelBuilder(
            backgroundLayer:GameObjectHolder,
            platformLayer:GameObjectHolder,
            enemiesLayer:GameObjectHolder,
            playerLayer:GameObjectHolder,
            itemLayer:GameObjectHolder )
        {
            this.backgroundLayer = backgroundLayer;
            this.platformLayer   = platformLayer;
            this.enemiesLayer    = enemiesLayer;
            this.playerLayer     = playerLayer;
            this.itemLayer       = itemLayer;

            levelNumber = 0;
            leftSideBuildingFuncs = [
                LevelOne_Left,
                LevelTwo_Left,
                LevelThree_Left,
                LevelFour_Left ];
            rightSideBuildingFuncs = [
                LevelOne_Right,
                LevelTwo_Right,
                LevelThree_Right,
                LevelFour_Right ];
        }

        public function RecreateLeftSide():void
        {
            ClearLeftSideStuff();
            MakeStuff(leftSideBuildingFuncs);
        }

        public function RecreateRightSide():void
        {
            ClearRightSideStuff();
            MakeStuff(rightSideBuildingFuncs);

            // inc the number when the right side just changed
            // order should be
            //  make first left
            //  make first right
            //  repeat:
            //    left changes
            //    right changes
            ++levelNumber;
        }

        private function ClearLeftSideStuff():void
        {
            var layersToClear:/*GameObjectHolder*/Array = [
                backgroundLayer,
                platformLayer,
                enemiesLayer,
                itemLayer ];

            for each (var layer:GameObjectHolder in layersToClear)
            {
                for (var i:int = layer.Objects.length -1; i >= 0; --i )
                {
                    var go:GameObject = layer.Objects[i];
                    var bounds:Rectangle = go.GetCollisionBounds(layer.Container);
                    if (bounds.right <= 0.5 * G.GameScreenWidth)
                    {
                        layer.Remove(go);
                    }
                }
            }
        }

        private function ClearRightSideStuff():void
        {
            var layersToClear:/*GameObjectHolder*/Array = [
                backgroundLayer,
                platformLayer,
                enemiesLayer,
                itemLayer ];

            for each (var layer:GameObjectHolder in layersToClear)
            {
                for (var i:int = layer.Objects.length -1; i >= 0; --i )
                {
                    var go:GameObject = layer.Objects[i];
                    var bounds:Rectangle = go.GetCollisionBounds(layer.Container);
                    if (bounds.left >= 0.5 * G.GameScreenWidth)
                    {
                        layer.Remove(go);
                    }
                }
            }
        }

        private function MakeStuff(sideBuildingFuncs:/*Function*/Array):void
        {
            if (levelNumber < sideBuildingFuncs.length)
            {
                sideBuildingFuncs[levelNumber]();
            }
            else
            {
                Util.RandomOneOf(sideBuildingFuncs)();
            }
        }
        private function AddScreenEdgePieces_Left():void
        {
            AddFloor(   1, 17, 9 );
            AddFloor(   1, 18, 9 );
            AddFloor(   1, 19, 9 );
            AddFloor(   1, 20, 9 );
            AddFloor(   1, 21, 9 );
            AddFloor(   1, 0, 9, false );
            AddWall(    0, 0, 18 );
        }

        private function AddScreenEdgePieces_Right():void
        {
            AddFloor(   10, 17, 9 );
            AddFloor(   10, 18, 9 );
            AddFloor(   10, 19, 9 );
            AddFloor(   10, 20, 9 );
            AddFloor(   10, 21, 9 );
            AddFloor(   10, 0, 9, false );
            AddWall(    19, 0, 18 );
        }

        private function AddWall(xTile:int, yTile:int, heightInTiles:int):void
        {
            var block:Wall = platformLayer.Add(Wall) as Wall;
            block.x      = xTile * G.TILE_SIZE;
            block.y      = yTile * G.TILE_SIZE;
            block.height = heightInTiles * G.TILE_SIZE;
        }

        private function AddFloor(xTile:int, yTile:int, widthInTiles:int, canGoUpThrough:Boolean = true):void
        {
            var block:Floor = platformLayer.Add(Floor) as Floor;
            block.x = xTile  * G.TILE_SIZE;
            block.y = yTile  * G.TILE_SIZE;
            block.width = widthInTiles * G.TILE_SIZE;
            block.canGoUpThrough = canGoUpThrough;
        }

        private function AddNanner(xTile:int, yTile:int):void
        {
            var nanner:Nanner = itemLayer.Add(Nanner) as Nanner;
            nanner.x = (xTile + 0.5) * G.TILE_SIZE;
            nanner.y = (yTile + 0.5) * G.TILE_SIZE;
        }

        public function Shutdown():void
        {
        }

        /******************************************************************/
        private function LevelOne_Left():void
        {
            AddScreenEdgePieces_Left();

            //AddFloor( 6, 15, 4);

            // does not make a banana the first time
            if (0 != levelNumber)
                AddNanner(3, 16);
        }
        private function LevelOne_Right():void
        {
            AddScreenEdgePieces_Right();

            //AddFloor(10, 15, 4);

            AddNanner(17, 16);
        }
        private function LevelTwo_Left():void
        {
            AddScreenEdgePieces_Left();

            AddFloor(6, 16, 2, false);
            AddFloor(6, 15, 2, false);

            AddFloor(1, 13, 3, false);
            AddNanner(2, 11);
        }
        private function LevelTwo_Right():void
        {
            AddScreenEdgePieces_Right();

            AddWall(10, 15, 2);
            AddWall(13, 13, 2);

            AddFloor(16, 11, 3);
            AddNanner(17, 10);
        }
        private function LevelThree_Left():void
        {
            AddScreenEdgePieces_Left();

            AddWall(7, 11, 4);

            AddFloor(1, 15, 5, true);

            AddFloor(1, 13, 4, true);
            AddFloor(5, 13, 2, false);

            AddFloor(1, 11, 2, true);
            AddFloor(3, 11, 4, false);

            AddFloor(3, 9, 4);
            AddFloor(3, 7, 4);

            AddFloor(5, 5, 5);
            AddNanner(7, 3);
        }
        private function LevelThree_Right():void
        {
            AddScreenEdgePieces_Right();

            AddFloor(17, 15, 2);
            AddFloor(17, 13, 2);
            AddFloor(17, 11, 2);
            AddFloor(17,  9, 2);
            AddFloor(17,  7, 2);
            AddFloor(17,  5, 2);

            AddFloor(11,  9, 3, false);
            AddNanner(12, 8);
            AddFloor(11,  5, 3, false);
        }
        private function LevelFour_Left():void
        {
            AddScreenEdgePieces_Left();

            AddFloor(6, 15, 2);
            AddFloor(2, 13, 2);
            AddFloor(6, 11, 2);
            AddFloor(2,  9, 2);
            AddFloor(6,  7, 2);
            AddFloor(2,  5, 2);
            AddFloor(6,  3, 1);

            AddNanner(6, 2);
        }
        private function LevelFour_Right():void
        {
            AddScreenEdgePieces_Right();

            AddFloor(16, 11, 3);
            AddWall(14, 1, 8)

            AddNanner(17, 10);
        }
    }

}
