﻿package monkeykong.ingame
{
    import flash.display.DisplayObject;
    import flash.geom.Rectangle;
    import stego.core.GameObject;
    import monkeykong.Sounds;
    import stego.collision.CollisionInfo;
    import stego.constants.Keyboard;
    import stego.InputUtil;
    import stego.PlatformerCharacter;
    import stego.util.MathUtil;
    import stego.util.spriteanim.SpriteAnimUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class MarioClone extends PlatformerCharacter
    {
        [Embed(source='../../../assets/monkey/idle.png')]
        private static const IDLE:Class;
        [Embed(source='../../../assets/monkey/blink.png')]
        private static const BLINK:Class;
        private static const IDLE_FRAMES:Array = [IDLE, IDLE, IDLE, IDLE, IDLE, IDLE, IDLE, BLINK];

        [Embed(source='../../../assets/monkey/run2.png')]
        private static const RUN2:Class;
        [Embed(source='../../../assets/monkey/run4.png')]
        private static const RUN4:Class;
        [Embed(source='../../../assets/monkey/altrun_1_3.png')]
        private static const ALTRUN_1_3:Class;
        [Embed(source='../../../assets/monkey/altrun2.png')]
        private static const ALTRUN2:Class;
        [Embed(source='../../../assets/monkey/altrun4.png')]
        private static const ALTRUN4:Class;
        private static const RUN_FRAMES:Array = [RUN2, IDLE, RUN4, IDLE, ALTRUN2, ALTRUN_1_3, ALTRUN4, ALTRUN_1_3];

        [Embed(source='../../../assets/monkey/skid1.png')]
        private static const SKID1:Class;
        [Embed(source='../../../assets/monkey/skid2.png')]
        private static const SKID2:Class;
        private static const SKID_FRAMES:Array = [SKID1, SKID2];

        private static const JUMP_FRAMES:Array = [RUN2];
        [Embed(source='../../../assets/monkey/backflip.png')]
        private static const BACKFLIP:Class;
        private static const BACKFLIP_FRAMES:Array = [BACKFLIP];

        [Embed(source='../../../assets/monkey/lose.png')]
        private static const LOSE:Class;
        private static const LOSE_FRAMES:Array = [LOSE];

        /* this woudl be a cool refactoring to make happen for reals
        private var mControl:IPlayerControl;
        public function get Control():IPlayerControl { return mControl; }
        public function set Control(value:IPlayerControl):void { mControl = value; }*/

        private var _inputUtil:InputUtil;
        public function get inputUtil():InputUtil { return _inputUtil; }
        public function set inputUtil(value:InputUtil):void { _inputUtil = value; }

        // direction options
        protected static const LEFT:Number = -1;
        protected static const STOP:Number = 0;
        protected static const RIGHT:Number = 1;

        private var mAnimUtil:SpriteAnimUtil;

        private static const MOVE_RIGHT_KEY:Number = Keyboard.RIGHT;
        private static const MOVE_LEFT_KEY:Number = Keyboard.LEFT;
        private static const NUM_AIR_JUMPS:Number = 0;
        private static const PLAYER_JUMP_SPEED:Number = 95;
        private static const PLAYER_JUMP_STOP_ACCEL:Number = 0;
        private static const PLAYER_BACKFLIP_SPEED_X:Number = 40;
        private static const PLAYER_BACKFLIP_SPEED_Y:Number = 140;
        private static const REVERSE_DIRECTION_ACCEL:Number = 60;

        // how long it tkes to do the rotation for a backflip
        private static const BACKFLIP_ROT_TIME:Number = 0.75;

        private static const SKID_START_THRESHOLD:Number = 200.0;
        private static const SKID_STOP_THRESHOLD:Number = 1.0;

        private static const TIME_BETWEEN_FOOTSTEPS:Number = 0.15625;
        private var mFootStepTimer:Number;

        // keys you can use to jump
        private static const JUMP_KEYS:Array = [Keyboard.SPACE, Keyboard.SHIFT, Keyboard.UP, 90];

        // remember which key they used to jump, so we know which key's release event we care about
        private var mJumpKey:uint = 0;

        private var mIsJumping:Boolean = false;
        public function get IsJumping():Boolean { return mIsJumping; }
        public function set IsJumping(value:Boolean):void { mIsJumping = value; }

        private var mAirJumpsRemaining:Number = NUM_AIR_JUMPS;

        private var backflipRotationInProgress:Boolean;

        public function get skidding():Boolean { return -1 != mAnimUtil.curAnim.indexOf("skid");  }

        protected var _walkAccel:Number = 600;
        protected var _airAccel:Number = 600;
        protected var _airDecel:Number = 600;
        protected var _stopAccel:Number = 900;
        protected var _maxWalkSpeed:Number = 300;

        public function get walkAccel():Number { return _walkAccel; }
        public function set walkAccel(value:Number):void  { _walkAccel = value; }
        public function get airAccel():Number { return _airAccel; }
        public function set airAccel(value:Number):void  { _airAccel = value; }
        public function get airDecel():Number { return _airDecel; }
        public function set airDecel(value:Number):void  { _airDecel = value; }
        public function get stopAccel():Number { return _stopAccel; }
        public function set stopAccel(value:Number):void  { _stopAccel = value; }
        public function get maxWalkSpeed():Number { return _maxWalkSpeed; }
        public function set maxWalkSpeed(value:Number):void  { _maxWalkSpeed = value; }

        public override function Init():void
        {
            super.Init();

            walkAccel = 160;
            airAccel = 60;
            airDecel = 60;
            stopAccel = 160;
            maxWalkSpeed = 40;

            CreateGraphics();
            Restart();
        }

        protected function CreateGraphics():void
        {
            if (null == mAnimUtil)
            {
                mAnimUtil = new SpriteAnimUtil(this);
                mAnimUtil.AddAnimation("idle", IDLE_FRAMES, 2.0);
                mAnimUtil.AddAnimation("run", RUN_FRAMES, 16.0);
                mAnimUtil.AddAnimation("skid", SKID_FRAMES, 16.0);
                mAnimUtil.AddAnimation("skidturn", SKID_FRAMES, 16.0);
                mAnimUtil.AddAnimation("jump", JUMP_FRAMES);
                mAnimUtil.AddAnimation("fall", JUMP_FRAMES);
                mAnimUtil.AddAnimation("backflip", BACKFLIP_FRAMES);
                mAnimUtil.AddAnimation("lose", LOSE_FRAMES);
            }
        }

        /**
         * Get it ready to be reused again
         */
        public override function Restart():void
        {
            super.Restart();
            backflipRotationInProgress = false;
            EndBackflipRotation();
            mAnimUtil.curAnim = "idle";
            mFootStepTimer = 0;
        }

        /**
         * per-frame update
         * @param   elapsedTime - time since the last frame
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);
            mAnimUtil.Update(elapsedTime);

            if (PhysicsEnabled)
            {
                Walk(elapsedTime);
                Jumping(elapsedTime);
            }
        }

        /**
         * change dx based on user input
         */
        protected function Walk(elapsedTime:Number):void
        {
            var desiredDirection:Number = STOP;
            if (inputUtil.IsDown(MOVE_RIGHT_KEY))
            {
                desiredDirection = RIGHT;
            }
            else if (inputUtil.IsDown(MOVE_LEFT_KEY))
            {
                desiredDirection = LEFT;
            }

            DowalkAccel(desiredDirection, elapsedTime);

            // update ground animations now
            if (IsOnGround)
            {
                EndBackflipRotation();

                if ( STOP == desiredDirection )
                {
                    if ( Math.abs(dx) > SKID_START_THRESHOLD)
                    {
                        if (!skidding)
                        {
                            stego.Sounds.PlaySound(monkeykong.Sounds.skidSound, null, 0.25);
                        }
                        mAnimUtil.curAnim = "skid";
                    }
                    else if ( skidding && (Math.abs(dx) > SKID_STOP_THRESHOLD) )
                    {
                        // just keep skidding
                    }
                    else
                    {
                        mAnimUtil.curAnim = "idle";
                    }
                }
                else
                {
                    if ( ( ( RIGHT == desiredDirection ) && ( dx < 0.0 ) )
                            || ( ( LEFT == desiredDirection ) && ( dx > 0.0 ) ) )
                    {
                        if (!skidding)
                        {
                            stego.Sounds.PlaySound(monkeykong.Sounds.skidSound, null, 0.25);
                        }
                        mAnimUtil.curAnim = "skidturn";
                    }
                    else
                    {
                        mAnimUtil.curAnim = "run";
                    }
                }
            }

            Footsteps(elapsedTime);
        }

        /**
         * take care of making footstep noises when walking
         */
        private function Footsteps(elapsedTime:Number):void
        {
            if ("run" == mAnimUtil.curAnim)
            {
                mFootStepTimer += elapsedTime;
                if (mFootStepTimer >= TIME_BETWEEN_FOOTSTEPS)
                {
                    mFootStepTimer = 0;
                    stego.Sounds.PlaySound(monkeykong.Sounds.randomStepSound, null, 0.125);
                }
            }
        }

        /**
         *
         */
        protected function Jumping(elapsedTime:Number):void
        {
            if ( inputUtil.IsUp(mJumpKey) || IsOnGround)
            {
                this.IsJumping = false;
                mJumpKey = 0;
            }

            // if we aren't holding the jump button anymore, cut
            // off the jump height
            if (!IsOnGround && !this.IsJumping && (this.dy < 0) )
            {
                this.dy += PLAYER_JUMP_STOP_ACCEL * elapsedTime;

                // don't shoot us down, though
                if (this.dy > 0)
                {
                    this.dy = 0;
                }
            }

            // starting a new jump
            var justPressedJumpKey:uint = 0;
            for each (var keyCode:uint in JUMP_KEYS)
            {
                if (inputUtil.WasJustPressed(keyCode))
                {
                    justPressedJumpKey = keyCode;
                }
            }

            if (inputUtil.WasJustPressed(justPressedJumpKey))
            {
                if (IsOnGround || (mAirJumpsRemaining > 0) )
                {
                    this.IsJumping = true;
                    mJumpKey = justPressedJumpKey;

                    if (skidding)
                    {
                        this.dx = -PLAYER_BACKFLIP_SPEED_X * MathUtil.Sign(dx);
                        this.dy = -PLAYER_BACKFLIP_SPEED_Y;
                        stego.Sounds.PlaySound(monkeykong.Sounds.backflipSound);
                        mAnimUtil.curAnim = "backflip";
                        StartBackflipRotation();
                    }
                    else
                    {
                        this.dy = -PLAYER_JUMP_SPEED;
                        stego.Sounds.PlaySound(monkeykong.Sounds.jumpSound, null, 0.75);
                    }
                }
            }

            // reset air jumps after landing
            if (IsOnGround)
            {
                mAirJumpsRemaining = NUM_AIR_JUMPS;
            }
            else // jump animations!
            {
                if (!backflipRotationInProgress)
                {
                    mAnimUtil.curAnim = ( dy < 0 ) ? "jump" : "fall";
                }
            }
        }

        public function ResetAirJumps():void
        {
            mAirJumpsRemaining = NUM_AIR_JUMPS;
        }
        /**
         * Handle the math to do the right kinds of acceleration
         * for platform character movement
         */
        protected function DowalkAccel(desiredDirection:Number, elapsedTime:Number):void
        {
            var currentDirection:Number = 0;
            if (this.dx > 0)
            {
                currentDirection = RIGHT;
            }
            else if (this.dx < 0)
            {
                currentDirection = LEFT;
            }

            // controls a bit differently on the ground
            if (IsOnGround)
            {
                // changing direction or stopping
                if ( (currentDirection != 0) && (desiredDirection != currentDirection) )
                {
                    var stopAccel:Number = (0 == desiredDirection) ? _stopAccel : REVERSE_DIRECTION_ACCEL;
                    var ddx:Number = stopAccel * elapsedTime;

                    if ( ddx > Math.abs(this.dx) )
                    {
                        this.dx = 0;
                    }
                    else
                    {
                        if (this.dx > 0 )
                        {
                            this.dx -= ddx;
                        }
                        else
                        {
                            this.dx += ddx;
                        }
                    }
                }
                else // accelerating
                {
                    this.dx += desiredDirection * walkAccel * elapsedTime;
                }
            }
            else
            {
                if (STOP == desiredDirection)
                {
                    var delta_dx:Number = -MathUtil.Sign(dx) * airDecel * elapsedTime;
                    if (Math.abs(delta_dx) > Math.abs(dx))
                    {
                        dx = 0;
                    }
                    else
                    {
                        dx += delta_dx;
                    }
                }
                else
                {
                    this.dx += desiredDirection * _airAccel * elapsedTime;
                }
            }

            // max speed
            if (this.dx > _maxWalkSpeed)
            {
                this.dx = _maxWalkSpeed;
            }
            if (this.dx < -_maxWalkSpeed)
            {
                this.dx = -_maxWalkSpeed;
            }

            // flip graphics
            if (desiredDirection > 0)
            {
                this.scaleX = 1.0;
            }
            if (desiredDirection < 0)
            {
                this.scaleX = -1.0;
            }
        }


        private function StartBackflipRotation():void
        {
            dr = MathUtil.Sign(scaleX) * 360.0 / BACKFLIP_ROT_TIME;
            scheduler.Cancel(EndBackflipRotation);
            scheduler.CallOnce(BACKFLIP_ROT_TIME, EndBackflipRotation);
            backflipRotationInProgress = true;
        }

        private function EndBackflipRotation():void
        {
            backflipRotationInProgress = false;
            scheduler.Cancel(EndBackflipRotation);
            dr = 0;
            rotation = 0;
        }

        public override function CanPassThrough(ci:CollisionInfo, object:GameObject):Boolean
        {
            if (object is Floor)
            {
                var floor:Floor = object as Floor;
                if ( floor.canGoUpThrough && (ci.normal.y >= -Math.SQRT1_2) )
                {
                    return true;
                }
            }
            return super.CanPassThrough(ci, object);
        }

        override public function GetCollisionBounds(targetCoordinateSpace:DisplayObject = null):Rectangle
        {
            var bounds:Rectangle = super.GetCollisionBounds(targetCoordinateSpace);
            bounds.x += 4;
            bounds.width -= 8;
            return bounds;
        }
    }
}
