﻿package stego
{
    import flash.events.Event;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import stego.collision.CollisionInfo;
    import stego.core.GameObject;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PlatformerCharacter extends GameObject
    {
        public static const EVENT_DEATH:String = "death";

        private var mDying:Boolean;
        public function get Dying():Boolean { return mDying; }
        public function set Dying(value:Boolean):void { mDying = value; }

        private var mIsOnGround:Boolean;
        public function get IsOnGround():Boolean { return mIsOnGround; }

        /**
         * set initial values after spawning
         */
        public function Restart():void
        {
            mIsOnGround = false;
            PhysicsEnabled = true;
            mDying = false;
            this.dr = 0;
            this.rotation = 0;
            this.scaleX = 1;
            this.scaleY = 1;
        }

        /**
         * Start doing the dying animation
         */
        public function StartDeath():void
        {
            PhysicsEnabled = false;
            mDying = true;
            dispatchEvent(new Event(EVENT_DEATH));
        }

        /**
         * Setup that needs to happen before we do the collision checks
         * During collision checks, CollisionOccurred can get called many times.
         * Afterward, we go to PostCollisionCheck to make the actual movement happen.
         */
        public function PreCollisionCheck(timeDelta:Number):void
        {
            mIsOnGround = false;
        }

        /**
         *
         */
        public function PostCollisionCheck(timeDelta:Number):void
        {
        }

        /**
         * override to deal with collisions
         */
        public override function CollisionOccurred(ci:CollisionInfo, object:GameObject):void
        {
            super.CollisionOccurred(ci, object);

            mIsOnGround = mIsOnGround || (ci.normal.y < -Math.SQRT1_2);
        }

        /**
         * make the character face right
         */
        public function FaceRight():void
        {
            scaleX = Math.abs(scaleX);
        }

        /**
         * make the character face left
         */
        public function FaceLeft():void
        {
            scaleX = -Math.abs(scaleX);
        }
    }
}
