﻿package stego.core
{
    import flash.display.DisplayObjectContainer;
    import flash.display.Stage;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import stego.core.GameState;
    import stego.debug.DebugWidget;
    import stego.InputUtil;
    import stego.util.CheatUtil;
    import stego.util.music.MusicUtil;
    import stego.util.PhysicsUtil;
    import stego.util.SceneLayers;
    import stego.util.TimeUtil;

    /**
     * common interface to game implementations
     * @author Mike Meyer
     */
    public interface GameInterface
    {
        /**
         * The Stage that the game is attached to.
         */
        function get stage () : Stage

        /**see how much the game is zoomed in
         */
        function get Zoom():Number

        /**
         * set how much the game is zoomed in
         */
        function set Zoom(zoom:Number):void

        /**
         * "camera" position.  Where the _center_ of the screen is.
         */
        function get cameraX():Number
        function set cameraX(value:Number):void

        /**
         * "camera" position.  Where the _center_ of the screen is.
         */
        function get cameraY():Number
        function set cameraY(value:Number):void

        /**
         * Get and set the camera shake amount.
         * You want this to be zero most of the time!
         */
        function get CameraShake():Number
        function set CameraShake(value:Number):void

        /**
         * get and set the input utility for polling key/button states
         */
        function get inputUtil():InputUtil
        function set inputUtil(value:InputUtil):void

        /**
         * get and set the physics utility the game is using
         */
        function get Physics():PhysicsUtil
        function set Physics(value:PhysicsUtil):void

        /**
         * get how wide it thinks the stage is.
         * the actual stage size may change, but this will remain the same
         */
        function get StageWidth():Number

        /**
         * get how tall it thinks the stage is.
         * the actual stage size may change, but this will remain the same
         */
        function get StageHeight():Number

        /**
         * get the rect for what this thinks the stage is.
         * the actual stage size may change, but this will remain the same
         */
        function get StageRect():Rectangle

        /**
         * Get a reference to the music utility
         */
        function get Music():MusicUtil

        /**
         * Get a reference to the cheat utility
         */
        function get Cheats():CheatUtil

        /**
         * Get a reference to the SceneLayers utility
         */
        function get Layers():SceneLayers

        /**
         * Get a reference to the time utility
         */
        function get Time():TimeUtil

        /**
         * access to debugging features
         */
        function get Debug():DebugWidget

        /**
         * re-init.  Will get called automagically the first time.
         */
        function Init(stageWidth:Number, stageHeight:Number):void

        /**
         * freeze time and show the pause screen
         */
        function Pause():void

        /**
         * resume after pausing
         */
        function Unpause():void

        /**
         * set a script to switch to once the current update call is finished
         */
        function ChangeGameState(nextState:GameState):void

        /**
         * clean up anything this coul dpossibly leave dangling
         */
        function Shutdown():void
    }
}
