﻿package stego.script.step
{
    import stego.script.Step;

    /**
     * step to combine multiple steps into one
     * (to try and execute all on the same update call)
     * @author Mike Meyer
     */
    public class Parallel extends Step
    {
        private var mSteps:Array;
        private var mResults:Array;

        public function Parallel(steps:Array)
        {
            mSteps = steps.concat(); // make a copy
            mResults = new Array();
        }

        public override function Execute(timeDelta:Number):Boolean
        {
            var allReady:Boolean = true;

            for (var i:Number = 0; i < mSteps.length; ++i)
            {
                var step:Step = mSteps[i];

                // if it is the first time we are running
                // it, then we need to make sure
                // it loses any state information from the
                // last time through
                if (null == mResults[i])
                {
                    step.Restart();
                }

                if (true != mResults[i])
                {
                    mResults[i] = step.Execute(timeDelta);

                    if (!mResults[i])
                    {
                        allReady = false;
                    }
                }
            }

            // none of them blocked
            return allReady;
        }

        /**
         * Go back to the beginning
         */
        public override function Restart():void
        {
            for (var i:Number = 0; i < mSteps.length; ++i)
            {
                mSteps[i].Restart();
            }

            mResults = new Array();
        }
    }

}
