package stego.util
{
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import flash.ui.ContextMenuBuiltInItems;
    import stego.core.GameObject;
    import stego.core.GameObjectHolder;
    import stego.Settings;
    import stego.widget.InputText;
    import stego.widget.PulsingTextButton;
    import stego.widget.TextButton;
    import stego.widget.TextField;

    /**
     * Utility for putting simple guis together easily
     * @author Mike Meyer
     */
    public class GuiBuilder
    {
        private var _font:String;
        public function get Font():String { return _font; }
        public function set Font(value:String):void { _font = value; }

        private var _fontColor:Number;
        public function get FontColor():Number { return _fontColor; }
        public function set FontColor(value:Number):void { _fontColor = value; }

        private var _highlightColor:Number;
        public function get HighlightColor():Number { return _highlightColor; }
        public function set HighlightColor(value:Number):void { _highlightColor = value; }

        private var _fontSize:Number;
        public function get FontSize():Number { return _fontSize; }
        public function set FontSize(value:Number):void { _fontSize = value; }

        private var _layer:GameObjectHolder;
        public function get Layer():GameObjectHolder { return _layer; }
        public function set Layer(value:GameObjectHolder):void { _layer = value; }

        private var mArea:Rectangle;

        function GuiBuilder(usableArea:Rectangle, layer:GameObjectHolder)
        {
            _font = Settings.Font;
            _fontColor = Settings.FontColor;
            _fontSize = Settings.FontSize;
            _highlightColor = Settings.HighlightColor;
            _layer = layer;
            mArea = usableArea;
        }

        /**
         * helper for the add text functions
         * handles everything but the positioning
         */
        private function _AddText(text:String):TextField
        {
            var tf:TextField = _layer.Add(TextField) as TextField;
            tf.SetText(text);
            tf.SetFont(Font);
            tf.SetFontColor(FontColor);
            tf.SetFontSize(FontSize);
            return tf;
        }

        /**
         * add an uncentered button to the layer
         * @param	text     text to display in it
         * @param	callback function to get called when the button is clicked
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function AddButton(text:String, callback:Function, x_pct:Number, y_pct:Number):TextButton
        {
            var b:TextButton = _layer.Add(TextButton) as TextButton;
            b.SetText(text);
            b.SetFont(Font);
            b.SetFontColor(FontColor);
            b.SetFontSize(FontSize);
            b.SetHoverColor(HighlightColor);
            b.addEventListener(MouseEvent.CLICK, callback);
            b.x = mArea.left + (mArea.width * x_pct);
            b.y = mArea.top + (mArea.height * y_pct);
            return b;
        }

        /**
         * Add a button to the layer
         * @param	text    text to display in it
         * @param	callback function to get called when the button is clicked
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function AddButtonCentered(text:String, callback:Function, x_pct:Number, y_pct:Number):TextButton
        {
            var b:TextButton = AddButton(text, callback, 0, 0);
            b.CenterOn( mArea.left + (mArea.width * x_pct),
                        mArea.top  + (mArea.height * y_pct) );
            return b;
        }

        /**
         * add an uncentered button to the layer
         * @param	text     text to display in it
         * @param	callback function to get called when the button is clicked
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function AddPulsingButton(text:String, callback:Function, x_pct:Number, y_pct:Number):PulsingTextButton
        {
            var b:PulsingTextButton = _layer.Add(PulsingTextButton) as PulsingTextButton;
            b.SetText(text);
            b.SetFont(Font);
            b.SetFontColor(FontColor);
            b.SetFontSize(FontSize);
            b.SetHoverColor(HighlightColor);
            b.addEventListener(MouseEvent.CLICK, callback);
            b.x = mArea.left + (mArea.width * x_pct);
            b.y = mArea.top + (mArea.height * y_pct);
            return b;
        }

        /**
         * Add a button to the layer
         * @param	text    text to display in it
         * @param	callback function to get called when the button is clicked
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function AddPulsingButtonCentered(text:String, callback:Function, x_pct:Number, y_pct:Number):PulsingTextButton
        {
            var b:PulsingTextButton = AddPulsingButton(text, callback, 0, 0);
            b.CenterOn( mArea.left + (mArea.width * x_pct),
                        mArea.top  + (mArea.height * y_pct) );
            return b;
        }

        /**
         * Add a textfield to the layer
         * @param	text    text to display in it
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function AddText(text:String, x_pct:Number, y_pct:Number):TextField
        {
            var tf:TextField = _AddText(text);
            tf.x = mArea.left + (mArea.width * x_pct);
            tf.y = mArea.top + (mArea.height * y_pct);
            return tf;
        }

        /**
         * Add a textfield to the layer
         * @param	text    text to display in it
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function AddTextCentered(text:String, x_pct:Number, y_pct:Number):TextField
        {
            var tf:TextField = _AddText(text);
            tf.CenterOn( mArea.left + (mArea.width * x_pct),
                         mArea.top + (mArea.height * y_pct) );
            return tf;
        }

        /**
         * Add a scrim to darken anything in back of it and make foreground elements stand out
         * @param	opacity
         * @param	color
         * @return
         */
        public function AddScrim(opacity:Number, color:Number = 0x000000):GameObject
        {
            var go:GameObject = _layer.Add();
            go.graphics.beginFill(color);
            go.graphics.drawRect(
                mArea.left,
                mArea.top,
                mArea.width,
                mArea.height);
            go.graphics.endFill();
            go.alpha = opacity;
            return go;
        }

        /**
         * Add an inputtext widget to the layer
         * @param	text    text to display in it
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function AddInputText(text:String, x_pct:Number, y_pct:Number):InputText
        {
            var it:InputText = _layer.Add(InputText) as InputText;
            it.Text = text;
            it.Font = Font;
            it.FontColor = FontColor;
            it.FontSize = FontSize;
            it.x = mArea.left + (mArea.width * x_pct);
            it.y = mArea.top + (mArea.height * y_pct);
            return it;
        }
    }
}
