﻿package stego.util
{
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import stego.collision.CollisionInfo;
    import stego.core.GameObject;
    import stego.PlatformerCharacter;
    import stego.Util;
    import stego.util.SceneLayers;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PhysicsUtil
    {
        /**
         * used as a member var just temporarily to make it easier to
         * communicate this to ForEach functions
         */
        private var timeDelta:Number;

        /**
         * get/set the acceleration due to gravity in this game
         */
        public var gravity:Number = 2000;

        /**
         * get/set a maximum fall speed for this game
         */
        public var maxFallSpeed:Number = 600;

        /**
         * Run a physics step of the simulation
         */
        public function Update(layers:SceneLayers, timeDelta:Number):void
        {
            this.timeDelta = timeDelta;
            ApplyGravity(layers);
            Collisions(layers);
        }

        /**
         * detect and react to collisions
         */
        protected function Collisions(layers:SceneLayers):void
        {
            layers.playerLayer.ForEach(CallPreCollisionCheck);
            layers.npcLayer.ForEach(CallPreCollisionCheck);
            layers.enemyLayer.ForEach(CallPreCollisionCheck);
            layers.itemLayer.ForEach(CallPreCollisionCheck);

            SceneLayers.ForEachCartesianProductStyle(layers.playerLayer, layers.platformLayer, CharacterPlatformCollision);
            SceneLayers.ForEachCartesianProductStyle(layers.playerLayer, layers.itemLayer, CharacterPlatformCollision);
            SceneLayers.ForEachCartesianProductStyle(layers.npcLayer, layers.platformLayer, CharacterPlatformCollision);
            SceneLayers.ForEachCartesianProductStyle(layers.enemyLayer, layers.platformLayer, CharacterPlatformCollision);
            SceneLayers.ForEachCartesianProductStyle(layers.itemLayer, layers.platformLayer, CharacterPlatformCollision);

            layers.playerLayer.ForEach(CallPostCollisionCheck);
            layers.npcLayer.ForEach(CallPostCollisionCheck);
            layers.enemyLayer.ForEach(CallPostCollisionCheck);
            layers.itemLayer.ForEach(CallPostCollisionCheck);
        }

        /**
         */
        protected function CallPreCollisionCheck(go:GameObject):void
        {
            if (go is PlatformerCharacter)
            {
                var character:PlatformerCharacter = go as PlatformerCharacter;
                character.PreCollisionCheck(timeDelta);
            }
        }
        /**
         */
        protected function CallPostCollisionCheck(go:GameObject):void
        {
            if (go is PlatformerCharacter)
            {
                var character:PlatformerCharacter = go as PlatformerCharacter;
                character.PostCollisionCheck(timeDelta);
            }
        }

        /**
         * detect and react to the player colliding with platforms
         */
        protected function CharacterPlatformCollision(go:GameObject, platform:GameObject):void
        {
            if (go.PhysicsEnabled && platform.PhysicsEnabled)
            {
                var ci:CollisionInfo = Util.CheckAabbCollision(go, platform, timeDelta);

                if (null != ci)
                {
                    var converted:CollisionInfo = ci.GlobalToLocal(go.parent);

                    // let the character know he hit something
                    go.CollisionOccurred(converted, platform);
                }
            }
        }

        /**
         * everything that can fall needs to have gravity applied
         */
        protected function ApplyGravity(layers:SceneLayers):void
        {
            layers.npcLayer.ForEach(ApplyGravityTo);
            layers.playerLayer.ForEach(ApplyGravityTo);
            layers.enemyLayer.ForEach(ApplyGravityTo);
            layers.bloodLayer.ForEach(ApplyGravityTo);
            layers.itemLayer.ForEach(ApplyGravityTo);
        }

        /**
         * apply gravity to the object
         */
        protected function ApplyGravityTo(go:GameObject):void
        {
            if (go.AffectedByGravity)
            {
                go.dy += gravity * timeDelta;

                if (go.dy > maxFallSpeed)
                {
                    go.dy = maxFallSpeed;
                }
            }
        }
    }

}
