﻿package stego.util.schedule
{
    import flash.display.DisplayObject;
    import stego.script.Step;
    import stego.script.step.Animate;
    import stego.util.ArrayUtil;

    /**
     * keep track of things that are supposed to happen in the future and make
     * sure they happen when they are supposed to
     * @author Mike Meyer
     */
    public class Schedule
    {
        protected var scheduledItems:Array = [];

        public function CallEvery(delay:Number, callback:Function):void
        {
            scheduledItems.push(new ScheduleItem(delay, callback, true));
        }

        public function CallOnce(delay:Number, callback:Function):void
        {
            scheduledItems.push(new ScheduleItem(delay, callback, false));
        }

        public function RunScript(step:Step):void
        {
            scheduledItems.push(new ScriptScheduleItem(step));
        }

        public function WatchCountdown(delay:Number, callback:Function):void
        {
            scheduledItems.push(new CountdownScheduleItem(delay, callback));
        }

        /**
         * remove a function from the schedule
         */
        public function Cancel(callback:Function):void
        {
            var predicate:Function = function(scheduleItem:IScheduleItem):Boolean { return (scheduleItem.callback == callback); };
            ArrayUtil.RemovePredicate(scheduledItems, predicate );
        }

        /**
         * per-frame update
         * @param	elapsedTime - time since the last update (in seconds)
         */
        public function Update(elapsedTime:Number):void
        {
            for (var i:int = scheduledItems.length - 1; i >= 0 ; --i)
            {
                var scheduleItem:IScheduleItem = scheduledItems[i] as IScheduleItem;
                scheduleItem.Update(elapsedTime);
                if (scheduleItem.isDone)
                {
                    scheduledItems.splice(i, 1);
                }
            }
        }

        /**
         * clean up
         */
        public function Shutdown():void
        {
            scheduledItems = [];
        }
    }

}
