﻿package stego.widget
{
    import flash.events.Event;
    import flash.events.TextEvent;
    import flash.text.TextFieldAutoSize;
    import flash.text.TextFieldType;
    import flash.text.TextFormat;
    import stego.core.GameObject;
    import stego.Settings;

    /**
     * wrapper around flash textfields for acting something like a standard windows textfield
     * Supports the same Event.CHANGE event that flash textfields do.
     * @author Mike Meyer
     */
    public class InputText extends GameObject
    {
        /**
         * actual textfield to wrap
         */
        protected var mTextField:flash.text.TextField;

        private static const DEFAULT_WORDWRAP_WIDTH:Number = 200;

        public function get WordWrap():Boolean { return mTextField.wordWrap; }
        public function set WordWrap(value:Boolean ):void { mTextField.wordWrap = value; }

        public function get WordWrapWidth():Number { return mTextField.width; }
        public function set WordWrapWidth(value:Number):void { mTextField.width = value; }

        /**
         * get/set the displayed text.
         */
        public function get Text():String { return mTextField.text; }
        public function set Text(desiredText:String):void { mTextField.text = desiredText; }

        /**
         * get/set the displayed text.
         */
        public function get HtmlText():String { return mTextField.htmlText; }
        public function set HtmlText(desiredHtmlText:String):void{ mTextField.htmlText = desiredHtmlText; }

        /**
         * get/set whether the textfield allows newlines
         */
        public function get Multiline():Boolean { return mTextField.multiline; }
        public function set Multiline(value:Boolean):void{ mTextField.multiline = value; }


        /**
         * get/set the font.
         */
        public function get Font():String
        {
            var fmt:TextFormat = mTextField.getTextFormat();
            return fmt.font;
        }
        public function set Font(fontName:String):void
        {
            var fmt:TextFormat = mTextField.getTextFormat();
            fmt.font = fontName;
            mTextField.defaultTextFormat = fmt;
            mTextField.setTextFormat(fmt);
        }

        /**
         * get/set the font size.
         */
        public function get FontSize():Number
        {
            var fmt:TextFormat = mTextField.getTextFormat();
            return Number(fmt.size);
        }
        public function set FontSize(desiredSize:Number):void
        {
            var fmt:TextFormat = mTextField.getTextFormat();
            fmt.size = desiredSize;
            mTextField.defaultTextFormat = fmt;
            mTextField.setTextFormat(fmt);
        }

        /**
         * get/set the color of the displayed text
         */
        public function get FontColor():Number { return mTextField.textColor; }
        public function set FontColor(color:Number):void { mTextField.textColor = color; }


        public function InputText():void
        {
            super();

            if (null == mTextField)
            {
                mTextField = new flash.text.TextField();
                mTextField.autoSize = TextFieldAutoSize.LEFT;
                mTextField.multiline = false;
                mTextField.type = TextFieldType.INPUT;
                mTextField.addEventListener(Event.CHANGE, TextInputHandler);
                this.addChild(mTextField);
            }
        }

        /**
         * Just relay the event along
         */
        private function TextInputHandler(e:Event):void
        {
            dispatchEvent(e);
        }

        /**
         * this function called for both initial setup and to re-init for reuse
         */
        public override function Init():void
        {
            super.Init();
            ResetDefaults();
        }

        /**
         * reset to defaults.
         */
        private function ResetDefaults():void
        {
            mTextField.x = 0;
            mTextField.y = 0;
            mTextField.selectable = true;
            mTextField.embedFonts = true;

            var format:TextFormat = new TextFormat();
            format.font = Settings.Font;
            format.size = Settings.FontSize;
            mTextField.defaultTextFormat = format;
            mTextField.setTextFormat(format);
            mTextField.textColor = Settings.FontColor;
            mTextField.text = "";

            WordWrap = false;
        }

        /**
         * Give Flash's focus to the textfield
         */
        public function GrabFocus():void
        {
            stage.focus = mTextField;
        }

        /**
         * Select everything so that when they start typing it gets replaced
         */
        public function SelectAll():void
        {
            mTextField.setSelection(0, mTextField.text.length);
        }
    }
}
