﻿package stego.widget
{
    import flash.display.DisplayObject;
    import flash.display.DisplayObjectContainer;
    import flash.display.Sprite;
    import flash.geom.Rectangle;
    import stego.core.GameObject;
    import stego.util.MathUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PulsingGraphic extends GameObject
    {
        protected var mainGraphic:DisplayObject;
        protected var pulseGraphic:DisplayObject;
        protected var pulseHolder:DisplayObjectContainer;

        /**
         * Graphic to display for the button
         */
        public function get graphic():Class { return _buttonGraphic; }
        public function set graphic(value:Class):void
        {
            _buttonGraphic = value;
            SwapGraphic(_buttonGraphic);
        }

        /**
         * change the current graphic to graphicClass
         */
        protected function SwapGraphic(graphicClass:Class):void
        {
            if (null == pulseHolder)
            {
                pulseHolder = new Sprite();
                addChild(pulseHolder);
            }

            if (null != mainGraphic)
            {
                removeChild(mainGraphic);
            }
            mainGraphic = new graphicClass();
            addChild(mainGraphic);

            if (null != pulseGraphic)
            {
                pulseHolder.removeChild(pulseGraphic);
            }
            pulseGraphic = new graphicClass();
            pulseHolder.addChild(pulseGraphic);

            UpdatePulsePosition();
        }

        protected var _buttonGraphic:Class;

        private static const PULSE_MAX_SCALE:Number = 1.5;
        private static const PULSE_MAX_ALPHA:Number = 0.5;
        private static const PULSE_MIN_ALPHA:Number = 0.0;
        private static const PULSE_DURATION:Number  = 0.75;
        private static const PULSE_PERIOD:Number    = 1.0;

        private var pulseTimer:Number = 0;

        public override function Init():void
        {
            super.Init();
            scheduler.CallEvery(PULSE_PERIOD, Pulse);
        }

        public function Pulse():void
        {
            pulseTimer = 0;
        }

        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            pulseTimer += elapsedTime;

            var t:Number = Math.min(1.0, pulseTimer / PULSE_DURATION);
            pulseHolder.alpha = MathUtil.Lerp(PULSE_MAX_ALPHA, PULSE_MIN_ALPHA, t);
            pulseHolder.scaleX = MathUtil.Lerp( 1.0, PULSE_MAX_SCALE, t);
            pulseHolder.scaleY = pulseHolder.scaleX;
        }

        /**
         * keep the pulse centered behind the main graphic
         */
        private function UpdatePulsePosition():void
        {
            var bounds:Rectangle = mainGraphic.getBounds(this);
            pulseHolder.x = 0.5 * (bounds.left + bounds.right);
            pulseHolder.y = 0.5 * (bounds.top + bounds.bottom);
            pulseGraphic.x = -0.5 * (bounds.left + bounds.right);
            pulseGraphic.y = -0.5 * (bounds.top + bounds.bottom);
        }
    }

}
