﻿package stego.widget
{
    import flash.display.Sprite;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PulsingGraphicButton extends PulsingGraphic
    {
        private var mouseEventDummy:Sprite;

        /**
         * Graphic to display for the button when the mouse is not over it
         */
        public override function set graphic(value:Class):void
        {
            _buttonGraphic = value;
            if (!mouseIsOver)
            {
                SwapGraphic(_buttonGraphic);
            }
        }

        /**
         * Graphic to display for the button when the mouse is over it
         */
        public function get hoverGraphic():Class { return _hoverGraphic; }
        public function set hoverGraphic(value:Class):void
        {
            _hoverGraphic = value;

            if (mouseIsOver)
            {
                SwapGraphic(_hoverGraphic);
            }
        }
        protected var _hoverGraphic:Class;

        /**
         * whether the mouse is currently over the button
         */
        public function get mouseIsOver():Boolean { return _mouseIsOver; }
        public function set mouseIsOver(value:Boolean):void
        {
            _mouseIsOver = value;
            if (value && (null != hoverGraphic) )
            {
                SwapGraphic(hoverGraphic);
            }
            if (!value && (null != graphic))
            {
                SwapGraphic(graphic);
            }
        }
        protected var _mouseIsOver:Boolean = false;

        /**
         * initial setup/preparation for reuse
         */
        public override function Init():void
        {
            super.Init();

            if (null == mouseEventDummy)
            {
                mouseEventDummy = new Sprite();
                mouseEventDummy.graphics.beginFill(0, 0);
                mouseEventDummy.graphics.drawRect(0, 0, 1, 1);
                mouseEventDummy.graphics.endFill();
                this.addChild(mouseEventDummy);
                AddMouseEventListeners();
            }

            mouseIsOver = false;
        }

        /**
         * cleanup
         */
        public override function Shutdown():void
        {
            RemoveMouseEventListeners();
            super.Shutdown();
        }

        /**
         * change the current graphic to graphicClass
         */
        protected override function SwapGraphic(graphicClass:Class):void
        {
            super.SwapGraphic(graphicClass);
            removeChild(mouseEventDummy);
            addChild(mouseEventDummy);
            UpdateMouseEventDummy();
        }

        /**
         * make the dummy mouse event rect match the main graphic
         */
        private function UpdateMouseEventDummy():void
        {
            var bounds:Rectangle = mainGraphic.getBounds(mouseEventDummy.parent);
            mouseEventDummy.x = bounds.x;
            mouseEventDummy.y = bounds.y;
            mouseEventDummy.width = bounds.width;
            mouseEventDummy.height = bounds.height;
        }

        private function HandleRollOver(evt:MouseEvent = null):void
        {
            mouseIsOver = true;
        }

        private function HandleRollOut(evt:MouseEvent = null):void
        {
            mouseIsOver = false;
        }

        private function AddMouseEventListeners():void
        {
            mouseEventDummy.addEventListener(MouseEvent.ROLL_OVER, HandleRollOver);
            mouseEventDummy.addEventListener(MouseEvent.ROLL_OUT, HandleRollOut);
        }

        private function RemoveMouseEventListeners():void
        {
            mouseEventDummy.removeEventListener(MouseEvent.ROLL_OVER, HandleRollOver);
            mouseEventDummy.removeEventListener(MouseEvent.ROLL_OUT, HandleRollOut);
        }
    }

}
