﻿package stego.widget
{
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import flash.media.Sound;
    import flash.media.SoundChannel;
    import stego.constants.Colors;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PulsingTextButton extends PulsingText
    {
        import flash.media.Sound;
        import flash.media.SoundChannel;

        private static var SoundCh:SoundChannel;

        private var mHoverColor:Number = Colors.YELLOW;
        private var mMouseEventDummy:Sprite;

        public override function Init():void
        {
            super.Init();

            AddMouseEventListeners();
        }

        protected override function CreateGraphics():void
        {
            super.CreateGraphics();

            if (null == mMouseEventDummy)
            {
                mMouseEventDummy = new Sprite();
                mMouseEventDummy.graphics.beginFill(0, 0);
                mMouseEventDummy.graphics.drawRect(0, 0, 1, 1);
                mMouseEventDummy.graphics.endFill();
                this.addChild(mMouseEventDummy);
            }

            MatchDummyToText();
        }

        public function SetHoverColor(color:Number):void
        {
            mHoverColor = color;
        }

        public override function Shutdown():void
        {
            RemoveMouseEventListeners();
            super.Shutdown();
        }

        private function AddMouseEventListeners():void
        {
            mMouseEventDummy.addEventListener(MouseEvent.ROLL_OVER, HandleRollOver);
            mMouseEventDummy.addEventListener(MouseEvent.ROLL_OUT, HandleRollOut);

            this.addEventListener(MouseEvent.CLICK, FixFocus);
        }

        private function RemoveMouseEventListeners():void
        {
            mMouseEventDummy.removeEventListener(MouseEvent.ROLL_OVER, HandleRollOver);
            mMouseEventDummy.removeEventListener(MouseEvent.ROLL_OUT, HandleRollOut);

            this.removeEventListener(MouseEvent.CLICK, FixFocus);
        }

        private function HandleRollOver(evt:MouseEvent = null):void
        {
            mTextField.SetFontColor(mHoverColor);
            mPulseTextField.SetFontColor(mHoverColor);
        }

        private function HandleRollOut(evt:MouseEvent = null):void
        {
            mTextField.SetFontColor(mColor);
            mPulseTextField.SetFontColor(mColor);
        }

        /**
         * workaround for weird flash issue with inputs after clicking a textfield
         */
        private function FixFocus(e:Event = null):void
        {
            // working around flash issue with clicked textfield making the keyboard not work.
            stage.focus = null;
        }

        /**
         * make the dummy mouse event rect match the displayed text
         */
        private function MatchDummyToText():void
        {
            var bounds:Rectangle = mTextField.getBounds(mMouseEventDummy.parent);
            mMouseEventDummy.x = bounds.x;
            mMouseEventDummy.y = bounds.y;
            mMouseEventDummy.width = bounds.width;
            mMouseEventDummy.height = bounds.height;
        }

        public override function CenterOn(x:Number, y:Number):void
        {
            super.CenterOn(x, y);
            MatchDummyToText();
        }
        public override function SetFont(font:String):void
        {
            super.SetFont(font);
            MatchDummyToText();
        }
        public override function SetFontSize(desiredSize:Number):void
        {
            super.SetFontSize(desiredSize);
            MatchDummyToText();
        }
        public override function SetText(text:String):void
        {
            super.SetText(text);
            MatchDummyToText();
        }
    }

}
