﻿package stego.widget
{
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import stego.constants.Colors;
    import stego.core.GameObject;

    /**
     * ...
     * @author Mike Meyer
     */
    public class TextButton extends GameObject
    {
        private var mHoverColor:Number = Colors.YELLOW;
        private var mMouseEventDummy:Sprite;
        private var mTextField:TextField;
        protected var mColor:Number = Colors.BLACK;

        public override function Init():void
        {
            super.Init();

            if (null == mTextField)
            {
                mTextField = new TextField();
                this.addChild(mTextField);
            }
            else
            {
                mTextField.ResetDefaults();
            }

            if (null == mMouseEventDummy)
            {
                mMouseEventDummy = new Sprite();
                mMouseEventDummy.graphics.beginFill(0, 0);
                mMouseEventDummy.graphics.drawRect(0, 0, 1, 1);
                mMouseEventDummy.graphics.endFill();
                this.addChild(mMouseEventDummy);
            }

            MatchDummyToText();

            AddMouseEventListeners();
        }

        public function SetHoverColor(color:Number):void
        {
            mHoverColor = color;
        }

        public override function Shutdown():void
        {
            RemoveMouseEventListeners();
            super.Shutdown();
        }

        private function AddMouseEventListeners():void
        {
            mMouseEventDummy.addEventListener(MouseEvent.ROLL_OVER, HandleRollOver);
            mMouseEventDummy.addEventListener(MouseEvent.ROLL_OUT, HandleRollOut);

            this.addEventListener(MouseEvent.CLICK, FixFocus);
        }

        private function RemoveMouseEventListeners():void
        {
            mMouseEventDummy.removeEventListener(MouseEvent.ROLL_OVER, HandleRollOver);
            mMouseEventDummy.removeEventListener(MouseEvent.ROLL_OUT, HandleRollOut);

            this.removeEventListener(MouseEvent.CLICK, FixFocus);
        }

        private function HandleRollOver(evt:MouseEvent = null):void
        {
            mTextField.SetFontColor(mHoverColor);
        }

        private function HandleRollOut(evt:MouseEvent = null):void
        {
            mTextField.SetFontColor(mColor);
        }

        /**
         * workaround for weird flash issue with inputs after clicking a textfield
         */
        private function FixFocus(e:Event = null):void
        {
            // working around flash issue with clicked textfield making the keyboard not work.
            stage.focus = null;
        }

        /**
         * make the dummy mouse event rect match the displayed text
         */
        private function MatchDummyToText():void
        {
            var bounds:Rectangle = mTextField.getBounds(mMouseEventDummy.parent);
            mMouseEventDummy.x = bounds.x;
            mMouseEventDummy.y = bounds.y;
            mMouseEventDummy.width = bounds.width;
            mMouseEventDummy.height = bounds.height;
        }

        public function CenterOn(x:Number, y:Number):void
        {
            this.x = x;
            this.y = y;
            mTextField.hAlign = TextField.HALIGN_CENTER;
            mTextField.vAlign = TextField.VALIGN_MIDDLE;
            MatchDummyToText();
        }
        public function SetFont(font:String):void
        {
            mTextField.SetFont(font);
            MatchDummyToText();
        }
        public function SetFontSize(desiredSize:Number):void
        {
            mTextField.SetFontSize(desiredSize);
            MatchDummyToText();
        }
        public function SetFontColor(color:Number):void
        {
            mColor = color;
            mTextField.SetFontColor(color);
        }
        public function SetText(text:String):void
        {
            mTextField.SetText(text);
            MatchDummyToText();
        }
    }

}
